library(brms)
library(tidyverse)
library(bayesplot)
library(ggrepel)
library(ggthemes)
bayesplot::color_scheme_set("darkgray")

# For reproducibility, we need to restrict the randomness a bit!
SEED <- 60511 # My oldest daughter was born 2016, June 11.
set.seed(SEED)

#Read the data into a dataframe.
y <- read.csv("exp0to2.csv", sep=";")
#Look at the data frame
glimpse(y)


#Create density plots for the raw data for each teask group A, B and C.
Group_A <- y[which(y$group == "A"),]
Group_B <- y[which(y$group == "B"),]
Group_C <- y[which(y$group == "C"),]
Group_A <- density(Group_A$estimate)
Group_B <- density(Group_B$estimate)
Group_C <- density(Group_C$estimate)



#Plot the densities for each task group.
plot(Group_A, ylim=c(0,0.4), xlim=c(0,15), 
     xlab="Estimates", main="", bty="n", lty=2)
lines(Group_B,lty=3)
lines(Group_C)
text(x = 3, y = 0.2, "A")
text(x = 6, y = 0.35, "B")
text(x = 10, y = 0.2, "C")
#Looks like we have some differences here. "A" seems to be the lower, "B" seems be a bit higher and "C"
#seems to be much higher, but with larger variance, but we need statistical inference.


#Now let do a sensitivity analysis of some models to see if they match with our assumptions. Note the 
#use of sample_prior="only", i.e., we are ignoring the likelihood now. We do NOT want our plot above 
#to influence our choice of priors so we always keep them around 0. The only thing we want to assure, 
#is that we models look reasonable and trunc crazy values.
#The first model is just the intercept. We do this because we want to see how a simple model compiles and then extend it.
M0.1 <- brm(formula = estimate ~ 1,
            data = y,
            prior = c(prior(normal(0, 5), class = Intercept),
                      prior(cauchy(0, 5), class = sigma)),#brm assumes it's half-cauchy for sigma.
            family = gaussian,
            seed = SEED,
            sample_prior = "only",
)

#We could create dummy variables for the categorical Group variable, 
#but we could just not have an intercept instead and include our variable "group" 
#as it is with the charaters as groups ("A", "B", and "C" in the raw data).
M0.2 <- brm(formula = estimate ~ 0 + group,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = gaussian,
          seed = SEED,
          sample_prior = "only",
)



#Since we have study_year as a variable too, we try including that one too. 
#Remember that we need to dummy code srudy_year too since it's a categorical variable
#In the raw data the study year is given by the numbers 2 (second year) and 3 (third year).
#We can therefore do the following:
y$year_dummy = (y$study_year - 2)

M0.3 <- brm(formula = estimate ~ 0 + group + year_dummy,
            data = y,
            prior = c(prior(normal(0, 1), class = b),
                      prior(cauchy(0, 5), class = sigma)),
            family = gaussian,
            seed = SEED,
            sample_prior = "only",
)

#We simulate estimations based on our models too see max and min values (are the models sane?).
s0.1 <- posterior_predict(M0.1)
max(s0.1, na.rm = T)
min(s0.1, na.rm = T)

s0.2 <- posterior_predict(M0.2)
max(s0.2, na.rm = T)
min(s0.2, na.rm = T)

s0.3 <- posterior_predict(M0.3)
max(s0.3, na.rm = T)
min(s0.3, na.rm = T)



#Al lthe models cover our reasonable values... but we cannot have negative values, 
#and it's not realistic to think we would obtain estimate over 100 weeks. We now include the likelihood (and only the prior too,), by
#choosing sample_prior = "yes".
#We can also throw our data at the priors to get our posterior and then do K-fold cross-validation (or Leave one out (LOO), in this case).
#But first, we need to create the posteriors. 
M3 <- brm(formula = estimate | trunc(lb=0, ub=100) ~ 0 + group,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = gaussian,
          seed = SEED,
          sample_prior = "yes",
)

M4 <- brm(formula = estimate | trunc(lb=0, ub=100) ~ 0 + group + year_dummy,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = gaussian,
          seed = SEED,
          sample_prior = "yes",
)
M3<-add_criterion(M3,"loo")
M4<-add_criterion(M4,"loo")

loo_compare(M3, M4)
#With study_year we don't get a lower elpd, so we pick M3 (study_year does not add predictive power).
M<-M3



#Rhat is also a way to check how chains mix. This value is value ok.
max(rhat(M))

#Now let's check if the predictions based on the model and the data (i.e. the posterior) fit the data.
ppc_dens_overlay(y=y$estimate, 
                 yrep=posterior_predict(M, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 
#Looks like an ok fit.


#Let's check how our chians mix visually.
plot(M)
# We see well-mixed chains, as should be the case.
#Converging chains should look like hairy caterpillars, which they do :) 


#Now let's look at our parameters to see if we have an effect. 
# Three density plots of alpha (Group A), beta_b (Group B) and beta_c (Group C). 
mcmc_areas(M, prob = 0.95, regex_pars = "^b") 
#Looks like all the groups are (very) significantly different from each other! As a comparison, even with overlapping 
#distributions, the differences are often significant if they are not too flat, which they aren't in this case.
#We can also look at the values for the plot:


M

#we mneed to check to that the difference in groups do not cross zero. We can see this in the plot above, but just to be safe.
#We can then take one posterior minus the another for another group. 
postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupB
dif <-density(dif)
plot(dif)
  

postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupC
dif <-density(dif)
plot(dif)

postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupC - postcompare$b_groupB
dif <-density(dif)
plot(dif)





#We are accostomed to looking at the overall effect size from a frequentist approach by looking at R^2. 
#We think it makes a lot of sense to see how much variance in the response that can be explained by the predictors. 
#Luckily there is a Bayesian alternative that is based sample draws from the posterior distribution when creating $R^2$ as a median.
#We should be careful here because the Bayaian def. of R^2 makes its use as a comparison of different models not recommended (see paper).
#We use LOO for model comparisons and BayesianR^2 as a measurement of explained variance by the model. 
median(bayes_R2(M))
#We can then see that 54% of the variance in estimations can be explained by Group.


#Let's now check how influential our priors are. 
Mtight<-
  update(M, prior = c(
    prior(normal(0, 0.1), class = b)),
    family = gaussian,
    seed = SEED,
    sample_prior = "yes",
    cores = 2
  )

Mwide<-
  update(M, prior = c(
    prior(normal(0, 10), class = b)),
    family = gaussian,
    seed = SEED,
    sample_prior = "yes",
    cores = 2
  )

#Our data swamps the prior in this case, which is good!
mcmc_areas(M, prob = 0.95, regex_pars = "^b|^prior_b") 
prior_summary(M)

#Now our priors really influebce our posterior, which is not what we want! 
#We don't want this informative prior if we don't base that on previous studies or expert input!
mcmc_areas(Mtight, prob = 0.95, regex_pars = "^b|^prior_b") 
prior_summary(Mtight)

#The prior is very very wide, but here the data overrules the prior, which is unncessary.. 
mcmc_areas(Mwide, prob = 0.95, regex_pars = "^b|^prior_b") 
prior_summary(Mwide)

mcmc_areas(M, prob = 0.95, regex_pars = "^s|^prior_s") 
#Our sigma very flat for our model and won't influence the result, which we settle with for now. 



