library(brms)
library(tidyverse)
library(bayesplot)
bayesplot::color_scheme_set("darkgray")

# For reproducibility.
SEED <- 60511 # My oldest daughter was born 2016, June 11.
set.seed(SEED)

#Read the data into a dataframe.
y <- read.csv("exp3.csv", sep=";")
#Look at the data frame
glimpse(y)


#Create density plots for the raw data for each teask group A, B and C.
Group_A <- y[which(y$group == "A"),]
Group_B <- y[which(y$group == "B"),]
Group_C <- y[which(y$group == "C"),]
Group_A <- density(Group_A$estimate)
Group_B <- density(Group_B$estimate)
Group_C <- density(Group_C$estimate)



#Plot the densities for each task group.
plot(Group_A, ylim=c(0,0.4), xlim=c(0,15), 
     xlab="Estimates", main="", bty="n", lty=2)
lines(Group_B,lty=3)
lines(Group_C)
text(x = 3.5, y = 0.2, "A")
text(x = 8, y = 0.35, "B")
text(x = 10, y = 0.2, "C")
#Looks like we have some differences here. "A" seems to be the lower, "B" seems be a bit higher and "C"
#seems to be much higher, but with larger variance. 


#Now let do a sensitivity analysis of some models to see if they match with our assumptions. 
M0.1 <- brm(formula = estimate ~ 1,
            data = y,
            prior = c(prior(normal(0, 5), class = Intercept),
                      prior(cauchy(0, 5), class = sigma)),
            family = gaussian,
            seed = SEED,
            sample_prior = "only",
            cores=2
)
#same as before.
M0.2 <- brm(formula = estimate ~ 0 + group,
            data = y,
            prior = c(prior(normal(0, 1), class = b),
                      prior(cauchy(0, 5), class = sigma)),
            family = gaussian,
            seed = SEED,
            sample_prior = "only",
            cores=2
)




#We simulate estimations based on our models too see max and min values (are the models sane?).
s0.1 <- posterior_predict(M0.1)
max(s0.1, na.rm = T)
min(s0.1, na.rm = T)

s0.2 <- posterior_predict(M0.2)
max(s0.2, na.rm = T)
min(s0.2, na.rm = T)





#Againwe cannot have negative values, and it's not realistic to think we would obtain estimate over 100 weeks. 
#Here we only have from the same study year so we cannot use that in our analysis.
M2 <- brm(formula = estimate | trunc(lb=0, ub=100) ~ 0 + group,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = gaussian,
          seed = SEED,
          sample_prior = "yes",
)


M<-M2



#Rhat is also a way to check how chains mix. This value is value ok.
max(rhat(M))

#Now let's check if the predictions based on the model and the data (i.e. the posterior) fit the data.
ppc_dens_overlay(y=y$estimate, 
                 yrep=posterior_predict(M, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 
#We are NOT happy with this fit. We therefore go back and try with a lognormal distribution. 



M3 <- brm(formula = estimate ~ 0 + group,
            data = y,
            prior = c(prior(normal(0, 1), class = b),
                      prior(cauchy(0, 5), class = sigma)),
            family = lognormal,
            seed = SEED,
            sample_prior = "yes",
)

M<-M3

max(rhat(M))

#Now let's check again.
ppc_dens_overlay(y=y$estimate, 
                 yrep=posterior_predict(M, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 
#Much better!!!



#Let's check how our chians mix visually.
plot(M)



#Now let's look at our parameters to see if we have an effect. 
# Three density plots of alpha (Group A), beta_b (Group B) and beta_c (Group C). 
mcmc_areas(M, prob = 0.95, regex_pars = "^b", transformations = "exp") 
#Looks like all the groups are (very) significantly different from each other! As a comparison, even with overlapping 
#distributions, the differences are often significant if they are not too flat, which they aren't in this case.
#We can also look at the values:


M
#Remember that we have a lognormal estimate valiable now so we need to do exp() on all the values in the model to create
#a table that matches the figure. Unfortunately we cannot do all these on a brm model directly... :(
mean_A <- exp(1.59) 
mean_B <- exp(2.06) 
mean_C <- exp(2.38) 
mean_sigma <- exp(0.17) 

mean_A
mean_B 
mean_C 
mean_sigma 

l95A <- exp(1.51)
l95B <- exp(1.99)
l95C <- exp(2.30)
l95sigma <- exp(0.14)

l95A 
l95B 
l95C 
l95sigma 

u95A <- exp(1.66)
u95B <- exp(2.14)
u95C <- exp(2.45)
u95sigma <- exp(0.21)

u95A 
u95B 
u95C 
u95sigma 


#we mneed to check to that the difference in groups do not cross zero. We can see this in the plot above, but just to be safe.
#We can then take one posterior minus the another for another group. 
postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupB
dif <-density(dif)
plot(dif)


postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupC
dif <-density(dif)
plot(dif)

postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupC - postcompare$b_groupB
dif <-density(dif)
plot(dif)



median(bayes_R2(M))

