library(brms)
library(tidyverse)
library(bayesplot)
library(ggplot2)
bayesplot::color_scheme_set("darkgray")

# For reproducibility.
SEED <- 60511 # My oldest daughter was born 2016, June 11.
set.seed(SEED)

#Read the data into a dataframe.
y <- read.csv("exp4.csv", sep=";")
#Look at the data frame
glimpse(y)


#Create density plots for the raw data for each teask group A, B and C.
Group_A <- y[which(y$group == "A"),]
Group_B <- y[which(y$group == "B"),]
Group_C <- y[which(y$group == "C"),]
Group_A <- density(Group_A$estimate)
Group_B <- density(Group_B$estimate)
Group_C <- density(Group_C$estimate)



#Plot the densities for each group.
plot(Group_A, ylim=c(0,0.5), xlim=c(0,15), 
     xlab="Estimates", main="", bty="n", lty=2)
lines(Group_B,lty=3)
lines(Group_C)
text(x = 3.5, y = 0.2, "A")
text(x = 8, y = 0.35, "B")
text(x = 10, y = 0.2, "C")
#Looks like we have some differences here again. "A" seems to be the lower, "B" seems be a bit higher and "C"
#seems to be much higher, but with larger variance. 


#Now let do a sensitivity analysis of some models
M0.1 <- brm(formula = estimate ~ 1,
            data = y,
            prior = c(prior(normal(0, 5), class = Intercept),
                      prior(cauchy(0, 5), class = sigma)),
            family = gaussian,
            seed = SEED,
            cores = 2,
            sample_prior = "only",
)

#We could create dummy variables for the categorical Group variable, but the zero below means no intercept.
M0.2 <- brm(formula = estimate ~ 0 + group,
            data = y,
            prior = c(prior(normal(0, 1), class = b),
                      prior(cauchy(0, 5), class = sigma)),
            family = gaussian,
            seed = SEED,
            cores = 2,
            sample_prior = "only",
)




#We simulate estimations based on our models too see max and min values (are the models sane?).
s0.1 <- posterior_predict(M0.1)
max(s0.1, na.rm = T)
min(s0.1, na.rm = T)

s0.2 <- posterior_predict(M0.2)
max(s0.2, na.rm = T)
min(s0.2, na.rm = T)





#We cannot have negative values, and it's not realistic to think we would obtain estimate over 100 weeks. 
M2 <- brm(formula = estimate | trunc(lb=0, ub=100) ~ 0 + group,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = gaussian,
          seed = SEED,
          sample_prior = "yes",
)


M<-M2



#Rhat is also a way to check how chains mix. This value is value ok.
max(rhat(M))

#Now let's check if the predictions based on the model and the data (i.e. the posterior) fit the data.
ppc_dens_overlay(y=y$estimate, 
                 yrep=posterior_predict(M, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 
#Again, we are NOT happy with this fit. We therefore go back and try with a lognormal distribution. 



M3 <- brm(formula = estimate ~ 0 + group,
          data = y,
          prior = c(prior(normal(0, 1), class = b),
                    prior(cauchy(0, 5), class = sigma)),
          family = lognormal,
          seed = SEED,
          sample_prior = "yes",
)



#Rhat is also a way to check how chains mix. This value is value ok.
max(rhat(M3))

#Now let's check if the predictions based on the model and the data (i.e. the posterior) fit the data.
ppc_dens_overlay(y=y$estimate, 
                 yrep=posterior_predict(M3, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 

M<-M3


#Let's check how our chians mix visually.
plot(M)
# We see well-mixed chains, as should be the case.
#Converging chains should look like hairy caterpillars, which they do :) 


#Now let's look at our parameters to see if we have an effect. 
# Three density plots of alpha (Group A), beta_b (Group B) and beta_c (Group C). 
ggplot <- mcmc_areas(M, prob = 0.95, pars = vars(b_groupA,b_groupB,b_groupC) , transformations = "exp") 
ggplot +  geom_vline(xintercept = 3.5)+ geom_vline(xintercept = 5, linetype = "longdash") + scale_x_continuous(limits=c(3,10))


#Looks like all the groups are significantly different from each other. As a comparison, even with overlapping 
#distributions, the differences are often significant if they are not too flat (and if they do not cross zero when stadnardized),
#which they aren't in this case.
#We can also look at the values:
M

#Just like before, we must take the exp() since we hve a lognormal response. 
mean_A <- exp(1.53) 
mean_B <- exp(1.90) 
mean_C <- exp(2.12) 
mean_sigma <- exp(0.22) 

mean_A
mean_B 
mean_C 
mean_sigma 


l95A <- exp(1.44)
l95B <- exp(1.82)
l95C <- exp(2.03)
l95sigma <- exp(0.19)

l95A 
l95B 
l95C 
l95sigma 

u95A <- exp(1.61)
u95B <- exp(1.99)
u95C <- exp(2.21)
u95sigma <- exp(0.26)

u95A 
u95B 
u95C 
u95sigma 


#we mneed to check to that the difference in groups do not cross zero. We can see this in the plot above, but just to be safe.
#We can then take one posterior minus the another for another group. 
postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupB
dif <-density(dif)
plot(dif)


postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupA - postcompare$b_groupC
dif <-density(dif)
plot(dif)


#This last one crosses zero but far out in its tail. 
postcompare<-posterior_samples(M)  
dif <- postcompare$b_groupC - postcompare$b_groupB
dif <-density(dif)
plot(dif)


median(bayes_R2(M))
#We can then see that 53% of the variance in estimations can be explained by Group, which is very high considering 
#so many other confounding factors when people make estimations of requirements!

